from kagglehub import registry
from tests.fixtures import BaseTestCase

SOME_VALUE = 1


def fail_fn(_):
    msg = "fail_fn should not be called"
    raise AssertionError(msg)


class FakeImpl:
    def __init__(self, is_supported_fn, call_fn):
        self._is_supported_fn = is_supported_fn
        self._call_fn = call_fn

    def is_supported(self, *args, **kwargs):
        return self._is_supported_fn(*args, **kwargs)

    def __call__(self, *args, **kwargs):
        return self._call_fn(*args, **kwargs)


class RegistryTest(BaseTestCase):
    def test_calls_only_supported(self):
        r = registry.MultiImplRegistry("test")
        r.add_implementation(FakeImpl(lambda _: True, lambda _: SOME_VALUE))
        r.add_implementation(FakeImpl(lambda _: False, fail_fn))

        val = r(SOME_VALUE)

        self.assertEqual(SOME_VALUE, val)

    def test_calls_first_supported_reverse(self):
        r = registry.MultiImplRegistry("test")
        r.add_implementation(FakeImpl(lambda _: True, fail_fn))
        r.add_implementation(FakeImpl(lambda _: True, lambda _: SOME_VALUE))

        val = r(SOME_VALUE)

        self.assertEqual(SOME_VALUE, val)

    def test_calls_throw_not_supported(self):
        r = registry.MultiImplRegistry("test")
        r.add_implementation(FakeImpl(lambda _: False, fail_fn))

        self.assertRaisesRegex(RuntimeError, r"Missing implementation", r, SOME_VALUE)



import os
import threading
from contextlib import contextmanager
from http.server import BaseHTTPRequestHandler, HTTPServer
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Type
from unittest import mock
from urllib.parse import urlparse

from kagglehub.clients import (
    KAGGLE_DATA_PROXY_TOKEN_ENV_VAR_NAME,
    KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME,
    KAGGLE_JWT_TOKEN_ENV_VAR_NAME,
)
from kagglehub.config import CACHE_FOLDER_ENV_VAR_NAME, KAGGLE_API_ENDPOINT_ENV_VAR_NAME
from kagglehub.kaggle_cache_resolver import KAGGLE_CACHE_MOUNT_FOLDER_ENV_VAR_NAME, KAGGLE_NOTEBOOK_ENV_VAR_NAME


def get_test_file_path(relative_path):
    return os.path.join(Path(__file__).parent, "data", relative_path)


@contextmanager
def create_test_cache():
    with TemporaryDirectory() as d:
        with mock.patch.dict(os.environ, {CACHE_FOLDER_ENV_VAR_NAME: d}):
            yield d


@contextmanager
def create_test_http_server(handler_class: Type[BaseHTTPRequestHandler]):
    endpoint = os.getenv(KAGGLE_API_ENDPOINT_ENV_VAR_NAME)
    test_server_address = urlparse(endpoint)
    if not test_server_address.hostname or not test_server_address.port:
        msg = f"Invalid test server address: {endpoint}. You must specify a hostname & port"
        raise ValueError(msg)
    with HTTPServer((test_server_address.hostname, test_server_address.port), handler_class) as httpd:
        threading.Thread(target=httpd.serve_forever).start()

        try:
            yield httpd
        finally:
            httpd.shutdown()


@contextmanager
def create_test_jwt_http_server(handler_class: Type[BaseHTTPRequestHandler]):
    with TemporaryDirectory() as cache_mount_folder:
        with mock.patch.dict(
            os.environ,
            {
                KAGGLE_NOTEBOOK_ENV_VAR_NAME: "Interactive",
                KAGGLE_JWT_TOKEN_ENV_VAR_NAME: "foo jwt token",
                KAGGLE_DATA_PROXY_TOKEN_ENV_VAR_NAME: "foo proxy token",
                KAGGLE_CACHE_MOUNT_FOLDER_ENV_VAR_NAME: cache_mount_folder,
            },
        ):
            endpoint = os.getenv(KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME)
            test_server_address = urlparse(endpoint)
            if not test_server_address.hostname or not test_server_address.port:
                msg = f"Invalid JWT test server address: {endpoint}. You must specify a hostname & port"
                raise ValueError(msg)
            with HTTPServer((test_server_address.hostname, test_server_address.port), handler_class) as httpd:
                threading.Thread(target=httpd.serve_forever).start()

                try:
                    yield httpd
                finally:
                    httpd.shutdown()



import json
import logging
import os
from typing import Tuple
from urllib.parse import urljoin

import requests
from requests.auth import HTTPBasicAuth
from tqdm import tqdm

from kagglehub.config import get_kaggle_api_endpoint, get_kaggle_credentials
from kagglehub.exceptions import BackendError, CredentialError, KaggleEnvironmentError

CHUNK_SIZE = 1048576
# The `connect` timeout is the number of seconds `requests` will wait for your client to establish a connection.
# The `read` timeout is the number of seconds that the client will wait BETWEEN bytes sent from the server.
# See: https://requests.readthedocs.io/en/stable/user/advanced/#timeouts
DEFAULT_CONNECT_TIMEOUT = 5  # seconds
DEFAULT_READ_TIMEOUT = 15  # seconds
ACCEPT_RANGE_HTTP_HEADER = "Accept-Ranges"

logger = logging.getLogger(__name__)


# TODO(b/307576378): When ready, use `kagglesdk` to issue requests.
class KaggleApiV1Client:
    BASE_PATH = "api/v1"

    def __init__(self):
        self.credentials = get_kaggle_credentials()
        self.endpoint = get_kaggle_api_endpoint()

    def get(self, path: str) -> dict:
        url = self._build_url(path)
        with requests.get(
            url,
            auth=self._get_http_basic_auth(),
            timeout=(DEFAULT_CONNECT_TIMEOUT, DEFAULT_READ_TIMEOUT),
        ) as response:
            response.raise_for_status()
            return response.json()

    def download_file(self, path: str, out_file: str):
        url = self._build_url(path)
        logger.info(f"Downloading from {url}...")
        with requests.get(
            url,
            stream=True,
            auth=self._get_http_basic_auth(),
            timeout=(DEFAULT_CONNECT_TIMEOUT, DEFAULT_READ_TIMEOUT),
        ) as response:
            response.raise_for_status()
            total_size = int(response.headers["Content-Length"])
            size_read = 0

            if _is_resumable(response) and os.path.isfile(out_file):
                size_read = os.path.getsize(out_file)

                logger.info(f"Resuming download from {size_read} bytes ({total_size - size_read} bytes left)...")

                # Send the request again with the 'Range' header.
                with requests.get(
                    response.url,  # URL after redirection
                    stream=True,
                    auth=self._get_http_basic_auth(),
                    timeout=(DEFAULT_CONNECT_TIMEOUT, DEFAULT_READ_TIMEOUT),
                    headers={"Range": f"bytes={size_read}-"},
                ) as resumed_response:
                    _download_file(resumed_response, out_file, size_read, total_size)
            else:
                _download_file(response, out_file, size_read, total_size)

    def _get_http_basic_auth(self):
        if self.credentials:
            return HTTPBasicAuth(self.credentials.username, self.credentials.key)
        return None

    def _build_url(self, path: str):
        return urljoin(self.endpoint, f"{KaggleApiV1Client.BASE_PATH}/{path}")


def _is_resumable(response: requests.Response):
    return ACCEPT_RANGE_HTTP_HEADER in response.headers and response.headers[ACCEPT_RANGE_HTTP_HEADER] == "bytes"


def _download_file(response: requests.Response, out_file: str, size_read: int, total_size: int):
    open_mode = "ab" if size_read > 0 else "wb"
    with tqdm(total=total_size, initial=size_read, unit="B", unit_scale=True, unit_divisor=1024) as progress_bar:
        with open(out_file, open_mode) as f:
            for chunk in response.iter_content(CHUNK_SIZE):
                f.write(chunk)
                size_read = min(total_size, size_read + CHUNK_SIZE)
                progress_bar.update(len(chunk))


# These environment variables are set by the Kaggle notebook environment.
KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME = "KAGGLE_DATA_PROXY_URL"
KAGGLE_JWT_TOKEN_ENV_VAR_NAME = "KAGGLE_USER_SECRETS_TOKEN"
KAGGLE_DATA_PROXY_TOKEN_ENV_VAR_NAME = "KAGGLE_DATA_PROXY_TOKEN"


class KaggleJwtClient:
    BASE_PATH = "/kaggle-jwt-handler/"

    def __init__(self):
        self.endpoint = os.getenv(KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME)
        if self.endpoint is None:
            msg = f"The {KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME} should be set."
            raise KaggleEnvironmentError(msg)
        jwt_token = os.getenv(KAGGLE_JWT_TOKEN_ENV_VAR_NAME)
        if jwt_token is None:
            msg = (
                "A JWT Token is required to call Kaggle, "
                f"but none found in environment variable {KAGGLE_JWT_TOKEN_ENV_VAR_NAME}"
            )
            raise CredentialError(msg)

        data_proxy_token = os.getenv(KAGGLE_DATA_PROXY_TOKEN_ENV_VAR_NAME)
        if data_proxy_token is None:
            msg = (
                "A Data Proxy Token is required to call Kaggle, "
                f"but none found in environment variable {KAGGLE_DATA_PROXY_TOKEN_ENV_VAR_NAME}"
            )
            raise CredentialError(msg)

        self.headers = {
            "Content-type": "application/json",
            "X-Kaggle-Authorization": f"Bearer {jwt_token}",
            "X-KAGGLE-PROXY-DATA": data_proxy_token,
        }

    def post(
        self,
        request_name: str,
        data: dict,
        timeout: Tuple[float, float] = (DEFAULT_CONNECT_TIMEOUT, DEFAULT_READ_TIMEOUT),
    ) -> dict:
        url = f"{self.endpoint}{KaggleJwtClient.BASE_PATH}{request_name}"
        with requests.post(
            url,
            headers=self.headers,
            data=bytes(json.dumps(data), "utf-8"),
            timeout=timeout,
        ) as response:
            response.raise_for_status()
            json_response = response.json()
            if "wasSuccessful" not in json_response:
                msg = "'wasSuccessful' field missing from response"
                raise BackendError(msg)
            if not json_response["wasSuccessful"]:
                msg = f"POST failed with: {response.text!s}"
                raise BackendError(msg)
            if "result" not in json_response:
                msg = "'result' field missing from response"
                raise BackendError(msg)
            return json_response["result"]



import json
import logging
import os
from tempfile import TemporaryDirectory
from unittest import mock

from kagglehub.config import (
    CACHE_FOLDER_ENV_VAR_NAME,
    CREDENTIALS_FILENAME,
    CREDENTIALS_FOLDER_ENV_VAR_NAME,
    DEFAULT_CACHE_FOLDER,
    DISABLE_KAGGLE_CACHE_ENV_VAR_NAME,
    KAGGLE_API_ENDPOINT_ENV_VAR_NAME,
    KEY_ENV_VAR_NAME,
    LOG_VERBOSITY_ENV_VAR_NAME,
    USERNAME_ENV_VAR_NAME,
    clear_kaggle_credentials,
    get_cache_folder,
    get_kaggle_api_endpoint,
    get_kaggle_credentials,
    get_log_verbosity,
    is_kaggle_cache_disabled,
    set_kaggle_credentials,
)
from tests.fixtures import BaseTestCase


class TestConfig(BaseTestCase):
    def test_get_cache_folder_default(self):
        self.assertEqual(DEFAULT_CACHE_FOLDER, get_cache_folder())

    def test_get_kaggle_api_endpoint_default(self):
        self.assertEqual("http://localhost:7777", get_kaggle_api_endpoint())

    @mock.patch.dict(os.environ, {KAGGLE_API_ENDPOINT_ENV_VAR_NAME: "http://localhost"})
    def test_get_kaggle_api_endpoint_environment_var_override(self):
        self.assertEqual("http://localhost", get_kaggle_api_endpoint())

    @mock.patch.dict(os.environ, {CACHE_FOLDER_ENV_VAR_NAME: "/test_cache"})
    def test_get_cache_folder_environment_var_override(self):
        self.assertEqual("/test_cache", get_cache_folder())

    def test_get_kaggle_credentials_not_set_returns_none(self):
        self.assertEqual(None, get_kaggle_credentials())

    @mock.patch.dict(os.environ, {USERNAME_ENV_VAR_NAME: "lastplacelarry"})
    def test_get_kaggle_credentials_missing_key_returns_none(self):
        self.assertEqual(None, get_kaggle_credentials())

    @mock.patch.dict(os.environ, {KEY_ENV_VAR_NAME: "some-key"})
    def test_get_kaggle_credentials_missing_username_returns_none(self):
        self.assertEqual(None, get_kaggle_credentials())

    @mock.patch.dict(os.environ, {USERNAME_ENV_VAR_NAME: "lastplacelarry", KEY_ENV_VAR_NAME: "some-key"})
    def test_get_kaggle_credentials_missing_username_succeeds(self):
        creds = get_kaggle_credentials()

        self.assertEqual("lastplacelarry", creds.username)
        self.assertEqual("some-key", creds.key)

    def test_get_kaggle_credentials_file_succeeds(self):
        with TemporaryDirectory() as d:
            with mock.patch.dict(os.environ, {CREDENTIALS_FOLDER_ENV_VAR_NAME: d}):
                with open(os.path.join(d, CREDENTIALS_FILENAME), "x") as creds_file:
                    json.dump({"username": "kerneler", "key": "another-key"}, creds_file)

                creds = get_kaggle_credentials()

                self.assertEqual("kerneler", creds.username)
                self.assertEqual("another-key", creds.key)

    def test_get_kaggle_credentials_invalid_json_file_raises(self):
        with TemporaryDirectory() as d:
            with mock.patch.dict(os.environ, {CREDENTIALS_FOLDER_ENV_VAR_NAME: d}):
                with open(os.path.join(d, CREDENTIALS_FILENAME), "x") as creds_file:
                    creds_file.write("invalid json credentials content")

                self.assertRaises(ValueError, get_kaggle_credentials)

    def test_get_kaggle_credentials_json_missing_username_raises(self):
        with TemporaryDirectory() as d:
            with mock.patch.dict(os.environ, {CREDENTIALS_FOLDER_ENV_VAR_NAME: d}):
                with open(os.path.join(d, CREDENTIALS_FILENAME), "x") as creds_file:
                    json.dump(
                        {
                            # Missing 'username'
                            "key": "another-key"
                        },
                        creds_file,
                    )

                self.assertRaises(ValueError, get_kaggle_credentials)

    def test_get_kaggle_credentials_json_missing_key_raises(self):
        with TemporaryDirectory() as d:
            with mock.patch.dict(os.environ, {CREDENTIALS_FOLDER_ENV_VAR_NAME: d}):
                with open(os.path.join(d, CREDENTIALS_FILENAME), "x") as creds_file:
                    json.dump(
                        {
                            "username": "kerneler",
                            # Missing 'key'
                        },
                        creds_file,
                    )

                self.assertRaises(ValueError, get_kaggle_credentials)

    def test_get_log_verbosity_default(self):
        self.assertEqual(logging.INFO, get_log_verbosity())

    @mock.patch.dict(os.environ, {LOG_VERBOSITY_ENV_VAR_NAME: "error"})
    def test_get_log_verbosity_environment_var_override(self):
        self.assertEqual(logging.ERROR, get_log_verbosity())

    @mock.patch.dict(os.environ, {LOG_VERBOSITY_ENV_VAR_NAME: "invalid"})
    def test_get_log_verbosity_environment_var_override_invalid_value_use_default(self):
        self.assertEqual(logging.INFO, get_log_verbosity())

    def test_is_kaggle_cache_disabled_default(self):
        # By default, the Kaggle cache is not disabled.
        self.assertFalse(is_kaggle_cache_disabled())

    def test_is_kaggle_cache_disabled(self):
        cases = [
            ("t", True),
            ("1", True),
            ("True", True),
            ("true", True),
            ("", False),
            ("0", False),
            ("False", False),
            ("false", False),
        ]
        for t in cases:
            env_var_value, expected = t[0], t[1]
            with mock.patch.dict(os.environ, {DISABLE_KAGGLE_CACHE_ENV_VAR_NAME: env_var_value}):
                self.assertEqual(expected, is_kaggle_cache_disabled())

    def test_set_kaggle_credentials_raises_error_with_whitespace(self):
        with self.assertRaises(ValueError):
            set_kaggle_credentials(username=" ", api_key="some-key")
        with self.assertRaises(ValueError):
            set_kaggle_credentials(username="lastplacelarry", api_key=" ")

    def test_set_and_clear_kaggle_credentials(self):
        # Set valid credentials
        set_kaggle_credentials("lastplacelarry", "some-key")

        # Get and assert credentials
        credentials = get_kaggle_credentials()
        self.assertEqual("lastplacelarry", credentials.username)
        self.assertEqual("some-key", credentials.key)

        # Clear credentials
        clear_kaggle_credentials()

        # Get and assert credentials are cleared
        credentials = get_kaggle_credentials()
        self.assertIsNone(credentials)



import kagglehub
from tests.fixtures import BaseTestCase


class TestModelUpload(BaseTestCase):
    def test_model_upload(self):
        with self.assertRaises(NotImplementedError):
            kagglehub.model_upload()



import json
import os
from http.server import BaseHTTPRequestHandler

import kagglehub
from kagglehub.cache import MODELS_CACHE_SUBFOLDER, get_cached_archive_path
from kagglehub.handle import parse_model_handle
from kagglehub.http_resolver import MODEL_INSTANCE_VERSION_FIELD
from tests.fixtures import BaseTestCase

from .utils import create_test_cache, create_test_http_server, get_test_file_path

INVALID_ARCHIVE_MODEL_HANDLE = "metaresearch/llama-2/pyTorch/bad-archive-variation/1"
VERSIONED_MODEL_HANDLE = "metaresearch/llama-2/pyTorch/13b/3"
UNVERSIONED_MODEL_HANDLE = "metaresearch/llama-2/pyTorch/13b"
TEST_FILEPATH = "config.json"


class KaggleAPIHandler(BaseHTTPRequestHandler):
    def do_HEAD(self):  # noqa: N802
        self.send_response(200)

    def do_GET(self):  # noqa: N802
        if self.path.endswith(VERSIONED_MODEL_HANDLE + "/download/" + TEST_FILEPATH):
            test_file_path = get_test_file_path(TEST_FILEPATH)
            with open(test_file_path, "rb") as f:
                # Serve a single file
                self.send_response(200)
                self.send_header("Content-type", "application/octet-stream")
                self.send_header("Content-Length", os.path.getsize(test_file_path))
                self.end_headers()
                self.wfile.write(f.read())
        elif self.path.endswith(VERSIONED_MODEL_HANDLE + "/download"):
            test_file_path = get_test_file_path("archive.tar.gz")
            with open(test_file_path, "rb") as f:
                # Serve archive file
                self.send_response(200)
                self.send_header("Content-type", "application/x-gzip")
                self.send_header("Content-Length", os.path.getsize(test_file_path))
                self.end_headers()
                self.wfile.write(f.read())
        elif self.path.endswith(INVALID_ARCHIVE_MODEL_HANDLE + "/download"):
            # Serve a bad archive file
            content = b"bad archive"
            self.send_response(200)
            self.send_header("Content-type", "application/x-gzip")
            self.send_header("Content-Length", len(content))
            self.end_headers()
            self.wfile.write(content)  # bad archive
        elif self.path.endswith(UNVERSIONED_MODEL_HANDLE + "/get"):
            self.send_response(200)
            self.send_header("Content-type", "application/json")
            self.end_headers()
            self.wfile.write(bytes(json.dumps({MODEL_INSTANCE_VERSION_FIELD: 3}), "utf-8"))
        else:
            self.send_response(404)
            self.wfile.write(bytes(f"Unhandled path: {self.path}", "utf-8"))


# Test cases for the HttpResolver.
class TestHttpModelDownload(BaseTestCase):
    def test_unversioned_model_download(self):
        with create_test_cache() as d:
            with create_test_http_server(KaggleAPIHandler):
                model_path = kagglehub.model_download(UNVERSIONED_MODEL_HANDLE)
                self.assertEqual(
                    os.path.join(d, MODELS_CACHE_SUBFOLDER, "metaresearch", "llama-2", "pyTorch", "13b", "3"),
                    model_path,
                )
                self.assertEqual(["config.json", "model.keras"], sorted(os.listdir(model_path)))

                # Assert that the archive file has been deleted.
                archive_path = get_cached_archive_path(parse_model_handle(UNVERSIONED_MODEL_HANDLE))
                self.assertFalse(os.path.exists(archive_path))

    def test_versioned_model_download(self):
        with create_test_cache() as d:
            with create_test_http_server(KaggleAPIHandler):
                model_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE)
                self.assertEqual(
                    os.path.join(d, MODELS_CACHE_SUBFOLDER, "metaresearch", "llama-2", "pyTorch", "13b", "3"),
                    model_path,
                )
                self.assertEqual(["config.json", "model.keras"], sorted(os.listdir(model_path)))

                # Assert that the archive file has been deleted.
                archive_path = get_cached_archive_path(parse_model_handle(VERSIONED_MODEL_HANDLE))
                self.assertFalse(os.path.exists(archive_path))

    def test_versioned_model_full_download_with_file_already_cached(self):
        with create_test_cache() as d:
            with create_test_http_server(KaggleAPIHandler):
                # Download a single file
                kagglehub.model_download(VERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)
                # Then download the full model and ensure all files are there.
                model_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE)

                self.assertEqual(
                    os.path.join(d, MODELS_CACHE_SUBFOLDER, "metaresearch", "llama-2", "pyTorch", "13b", "3"),
                    model_path,
                )
                self.assertEqual(["config.json", "model.keras"], sorted(os.listdir(model_path)))

    def test_unversioned_model_full_download_with_file_already_cached(self):
        with create_test_cache() as d:
            with create_test_http_server(KaggleAPIHandler):
                # Download a single file
                kagglehub.model_download(UNVERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)
                # Then download the full model and ensure all files are there.
                model_path = kagglehub.model_download(UNVERSIONED_MODEL_HANDLE)

                self.assertEqual(
                    os.path.join(d, MODELS_CACHE_SUBFOLDER, "metaresearch", "llama-2", "pyTorch", "13b", "3"),
                    model_path,
                )
                self.assertEqual(["config.json", "model.keras"], sorted(os.listdir(model_path)))

    def test_versioned_model_download_bad_archive(self):
        with create_test_cache():
            with create_test_http_server(KaggleAPIHandler):
                with self.assertRaises(ValueError):
                    kagglehub.model_download(INVALID_ARCHIVE_MODEL_HANDLE)

    def test_versioned_model_download_with_path(self):
        with create_test_cache() as d:
            with create_test_http_server(KaggleAPIHandler):
                model_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)
                self.assertEqual(
                    os.path.join(
                        d,
                        MODELS_CACHE_SUBFOLDER,
                        "metaresearch",
                        "llama-2",
                        "pyTorch",
                        "13b",
                        "3",
                        TEST_FILEPATH,
                    ),
                    model_path,
                )
                with open(model_path) as model_file:
                    self.assertEqual("{}", model_file.readline())

    def test_unversioned_model_download_with_path(self):
        with create_test_cache() as d:
            with create_test_http_server(KaggleAPIHandler):
                model_path = kagglehub.model_download(UNVERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)
                self.assertEqual(
                    os.path.join(
                        d,
                        MODELS_CACHE_SUBFOLDER,
                        "metaresearch",
                        "llama-2",
                        "pyTorch",
                        "13b",
                        "3",
                        TEST_FILEPATH,
                    ),
                    model_path,
                )
                with open(model_path) as model_file:
                    self.assertEqual("{}", model_file.readline())

    def test_versioned_model_download_already_cached(self):
        with create_test_cache() as d:
            # Download from server.
            with create_test_http_server(KaggleAPIHandler):
                kagglehub.model_download(VERSIONED_MODEL_HANDLE)

            # No internet, cache hit.
            model_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE)

            self.assertEqual(
                os.path.join(d, MODELS_CACHE_SUBFOLDER, "metaresearch", "llama-2", "pyTorch", "13b", "3"),
                model_path,
            )

    def test_versioned_model_download_with_path_already_cached(self):
        with create_test_cache() as d:
            with create_test_http_server(KaggleAPIHandler):
                kagglehub.model_download(VERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)

            # No internet, cache hit.
            model_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE, path=TEST_FILEPATH)

            self.assertEqual(
                os.path.join(
                    d, MODELS_CACHE_SUBFOLDER, "metaresearch", "llama-2", "pyTorch", "13b", "3", TEST_FILEPATH
                ),
                model_path,
            )



import os
from pathlib import Path

from kagglehub.cache import (
    MODELS_CACHE_SUBFOLDER,
    get_cached_archive_path,
    get_cached_path,
    load_from_cache,
    mark_as_complete,
)
from kagglehub.handle import ModelHandle
from tests.fixtures import BaseTestCase

from .utils import create_test_cache

TEST_MODEL_HANDLE = ModelHandle(
    owner="google",
    model="bert",
    framework="tensorFlow2",
    variation="answer-equivalence-bem",
    version=2,
)

TEST_FILEPATH = "foo.txt"


class TestCache(BaseTestCase):
    def test_load_from_cache_miss(self):
        ModelHandle(
            owner="google",
            model="bert",
            framework="tensorFlow2",
            variation="answer-equivalence-bem",
            version=2,
        )
        self.assertEqual(None, load_from_cache(TEST_MODEL_HANDLE))

    def test_load_from_cache_with_path_miss(self):
        self.assertEqual(None, load_from_cache(TEST_MODEL_HANDLE, TEST_FILEPATH))

    def test_load_from_cache_not_complete_miss(self):
        with create_test_cache():
            cache_path = get_cached_path(TEST_MODEL_HANDLE)

            # Should be a cache `miss` if the directory exists but not the marker file.
            os.makedirs(cache_path)

            self.assertEqual(None, load_from_cache(TEST_MODEL_HANDLE))

    def test_load_from_cache_with_path_not_complete_miss(self):
        with create_test_cache():
            cache_path = get_cached_path(TEST_MODEL_HANDLE)

            # Should be a cache `miss` if the directory exists but not the marker file.
            os.makedirs(cache_path)
            Path(os.path.join(cache_path, TEST_FILEPATH)).touch()  # Create file

            self.assertEqual(None, load_from_cache(TEST_MODEL_HANDLE, path=TEST_FILEPATH))

    def test_load_from_cache_with_complete_marker_no_files_miss(self):
        with create_test_cache():
            get_cached_path(TEST_MODEL_HANDLE)

            # Should be a cache `miss` if completion marker file exists but not the files themselves.
            mark_as_complete(TEST_MODEL_HANDLE)

            self.assertEqual(None, load_from_cache(TEST_MODEL_HANDLE))

    def test_load_from_cache_with_path_complete_marker_no_files_miss(self):
        with create_test_cache():
            get_cached_path(TEST_MODEL_HANDLE)

            # Should be a cache `miss` if completion marker file exists but not the file itself.
            mark_as_complete(TEST_MODEL_HANDLE, path=TEST_FILEPATH)

            self.assertEqual(None, load_from_cache(TEST_MODEL_HANDLE, path=TEST_FILEPATH))

    def test_cache_hit(self):
        with create_test_cache() as d:
            cache_path = get_cached_path(TEST_MODEL_HANDLE)
            os.makedirs(cache_path)
            mark_as_complete(TEST_MODEL_HANDLE)

            path = load_from_cache(TEST_MODEL_HANDLE)

            self.assertEqual(
                os.path.join(d, MODELS_CACHE_SUBFOLDER, "google", "bert", "tensorFlow2", "answer-equivalence-bem", "2"),
                path,
            )

    def test_cache_hit_with_path(self):
        with create_test_cache() as d:
            cache_path = get_cached_path(TEST_MODEL_HANDLE)

            os.makedirs(cache_path)
            Path(os.path.join(cache_path, TEST_FILEPATH)).touch()  # Create file
            mark_as_complete(TEST_MODEL_HANDLE, path=TEST_FILEPATH)

            path = load_from_cache(TEST_MODEL_HANDLE, path=TEST_FILEPATH)

            self.assertEqual(
                os.path.join(
                    d,
                    MODELS_CACHE_SUBFOLDER,
                    "google",
                    "bert",
                    "tensorFlow2",
                    "answer-equivalence-bem",
                    "2",
                    "foo.txt",
                ),
                path,
            )

    def test_load_from_cache_invalid_handle(self):
        with self.assertRaises(ValueError):
            load_from_cache("invalid_handle")

    def test_model_archive_path(self):
        with create_test_cache() as d:
            archive_path = get_cached_archive_path(TEST_MODEL_HANDLE)

            self.assertEqual(
                os.path.join(
                    d,
                    MODELS_CACHE_SUBFOLDER,
                    "google",
                    "bert",
                    "tensorFlow2",
                    "answer-equivalence-bem",
                    "2.archive",
                ),
                archive_path,
            )



import abc
from typing import Optional

from kagglehub.handle import ModelHandle


class Resolver:
    """Resolver base class: all resolvers inherit from this class."""

    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def __call__(self, handle: ModelHandle, path: Optional[str] = None) -> str:
        """Resolves a handle into a path with the requested model files.

        Args:
            handle: (string) the model handle to resolve.
            path: (string) Optional path to a file within the model bundle.


        Returns:
            A string representing the path
        """
        pass

    @abc.abstractmethod
    def is_supported(self, handle: ModelHandle, path: Optional[str] = None):
        """Returns whether the current environment supports this handle/path."""
        pass



from typing import Optional

from kagglehub import registry
from kagglehub.handle import parse_model_handle


def model_download(handle: str, path: Optional[str] = None):
    """Download model files.

    Args:
        handle: (string) the model handle.
        path: (string) Optional path to a file within the model bundle.

    Returns:
        A string representing the path to the requested model files.
    """
    h = parse_model_handle(handle)
    return registry.resolver(h, path)


def model_upload():
    raise NotImplementedError()



__version__ = "0.1.2"

import kagglehub.logging
from kagglehub import http_resolver, kaggle_cache_resolver, registry
from kagglehub.auth import login
from kagglehub.models import model_download, model_upload

registry.resolver.add_implementation(http_resolver.HttpResolver())
registry.resolver.add_implementation(kaggle_cache_resolver.KaggleCacheResolver())



import os
from pathlib import Path
from typing import Optional, Union

from kagglehub.config import get_cache_folder
from kagglehub.handle import ModelHandle

MODELS_CACHE_SUBFOLDER = "models"
MODELS_FILE_COMPLETION_MARKER_FOLDER = ".complete"


def load_from_cache(handle: Union[ModelHandle], path: Optional[str] = None) -> Optional[str]:
    """Return path for the requested resource from the cache.

    Args:
        handle: Resource handle
        path: Optional path to a file within the bundle.

    Returns:
        A string representing the path to the requested resource in the cache or None on cache miss.
    """
    marker_path = _get_completion_marker_filepath(handle, path)
    full_path = get_cached_path(handle, path)
    return full_path if os.path.exists(marker_path) and os.path.exists(full_path) else None


def get_cached_path(handle: Union[ModelHandle], path: Optional[str] = None) -> str:
    # Can extend to add support for other resources like DatasetHandle.
    if isinstance(handle, ModelHandle):
        return _get_model_path(handle, path)
    else:
        msg = "Invalid handle"
        raise ValueError(msg)


def get_cached_archive_path(handle: Union[ModelHandle]) -> str:
    if isinstance(handle, ModelHandle):
        return _get_model_archive_path(handle)
    else:
        msg = "Invalid handle"
        raise ValueError(msg)


def mark_as_complete(handle: Union[ModelHandle], path: Optional[str] = None):
    marker_path = _get_completion_marker_filepath(handle, path)
    os.makedirs(os.path.dirname(marker_path), exist_ok=True)
    Path(marker_path).touch()


def _get_completion_marker_filepath(handle: Union[ModelHandle], path: Optional[str] = None) -> str:
    # Can extend to add support for other resources like DatasetHandle.
    if isinstance(handle, ModelHandle):
        return _get_models_completion_marker_filepath(handle, path)
    else:
        msg = "Invalid handle"
        raise ValueError(msg)


def _get_model_path(handle: ModelHandle, path: Optional[str] = None) -> str:
    base_path = os.path.join(
        get_cache_folder(),
        MODELS_CACHE_SUBFOLDER,
        handle.owner,
        handle.model,
        handle.framework,
        handle.variation,
        str(handle.version),
    )

    return os.path.join(base_path, path) if path else base_path


def _get_model_archive_path(handle: ModelHandle) -> str:
    return os.path.join(
        get_cache_folder(),
        MODELS_CACHE_SUBFOLDER,
        handle.owner,
        handle.model,
        handle.framework,
        handle.variation,
        f"{handle.version!s}.archive",
    )


def _get_models_completion_marker_filepath(handle: ModelHandle, path: Optional[str] = None) -> str:
    if path:
        return os.path.join(
            get_cache_folder(),
            MODELS_CACHE_SUBFOLDER,
            handle.owner,
            handle.model,
            handle.framework,
            handle.variation,
            MODELS_FILE_COMPLETION_MARKER_FOLDER,
            str(handle.version),
            f"{path}.complete",
        )

    return os.path.join(
        get_cache_folder(),
        MODELS_CACHE_SUBFOLDER,
        handle.owner,
        handle.model,
        handle.framework,
        handle.variation,
        f"{handle.version!s}.complete",
    )
